package database;

/**
 * Structuration association proxy interface.
 */
public interface Structuration extends javax.jmi.reflect.RefAssociation {
    /**
     * Queries whether a link currently exists between a given pair of instance 
     * objects in the associations link set.
     * @param part Value of the first association end.
     * @param structure Value of the second association end.
     * @return Returns true if the queried link exists.
     */
    public boolean exists(database.DataType part, database.CompositeDataType structure);
    /**
     * Queries the instance objects that are related to a particular instance 
     * object by a link in the current associations link set.
     * @param structure Required value of the second association end.
     * @return Collection of related objects.
     */
    public java.util.Collection getPart(database.CompositeDataType structure);
    /**
     * Queries the instance object that is related to a particular instance object 
     * by a link in the current associations link set.
     * @param part Required value of the first association end.
     * @return Related object or <code>null</code> if none exists.
     */
    public database.CompositeDataType getStructure(database.DataType part);
    /**
     * Creates a link between the pair of instance objects in the associations 
     * link set.
     * @param part Value of the first association end.
     * @param structure Value of the second association end.
     */
    public boolean add(database.DataType part, database.CompositeDataType structure);
    /**
     * Removes a link between a pair of instance objects in the current associations 
     * link set.
     * @param part Value of the first association end.
     * @param structure Value of the second association end.
     */
    public boolean remove(database.DataType part, database.CompositeDataType structure);
}
