/*
 * $Id: MDRLogger.java,v 1.2 2004/02/16 15:44:25 dvojtise Exp $
 * Authors : ffondeme
 * Created on 1 avr. 2003
 * 
 * Copyright 2004 - INRIA - LGPL license 
 */
package ch.epfl.lgl.MDR;

import java.io.PrintStream;
import java.util.Date;
import java.util.Hashtable;
import java.util.WeakHashMap;

import org.openide.ErrorManager;

/**
 * @author ffondeme
 * Connects MDR logger to the driver logger
 */
public class MDRLogger extends ErrorManager {
	
	public static class Annotation implements ErrorManager.Annotation {
		protected final Date date;
		protected final String localizedMessage;
		protected final String message;
		protected final int severity;
		protected final Throwable stackTrace; 
		
		public Annotation(
			Throwable t,
			int severity,
			String message,
			String localizedMessage,
			Throwable stackTrace,
			Date date) {
			this.date = date;
			this.localizedMessage = localizedMessage;
			this.message = message;
			this.severity = severity;
			this.stackTrace = stackTrace;
		}
		
		/* (non-Javadoc)
		 * @see org.openide.ErrorManager.Annotation#getDate()
		 */
		public Date getDate() {
			return this.date;
		}

		/* (non-Javadoc)
		 * @see org.openide.ErrorManager.Annotation#getLocalizedMessage()
		 */
		public String getLocalizedMessage() {
			return this.localizedMessage;
		}

		/* (non-Javadoc)
		 * @see org.openide.ErrorManager.Annotation#getMessage()
		 */
		public String getMessage() {
			return this.message;
		}

		/* (non-Javadoc)
		 * @see org.openide.ErrorManager.Annotation#getSeverity()
		 */
		public int getSeverity() {
			return this.severity;
		}

		/* (non-Javadoc)
		 * @see org.openide.ErrorManager.Annotation#getStackTrace()
		 */
		public Throwable getStackTrace() {
			return this.stackTrace;
		}

		public String toString() {
			return "[" + this.getDate() + "] " + this.getMessage() + (this.getLocalizedMessage() == null ? "" : this.getLocalizedMessage()); 
		}
	}
	
	public static WeakHashMap instances = new WeakHashMap();

	protected String name;
	protected PrintStream out;
	protected PrintStream err;
	
	protected Hashtable annotations_throwable = new Hashtable();

	
	public MDRLogger(String name) {
		this(name, System.out, System.err);
	}
	
	public MDRLogger(String name, PrintStream out, PrintStream err) {
		this.name = name;
		this.out = out;
		this.err = err;
		instances.put(name, this);
	}

	/* (non-Javadoc)
	 * @see org.openide.ErrorManager#attachAnnotations(java.lang.Throwable, org.openide.ErrorManager.Annotation[])
	 */
	public Throwable attachAnnotations(Throwable t, ErrorManager.Annotation[] arr) {
		this.annotations_throwable.put(t, arr);
		return t;
	}

	/* (non-Javadoc)
	 * @see org.openide.ErrorManager#findAnnotations(java.lang.Throwable)
	 */
	public ErrorManager.Annotation[] findAnnotations(Throwable t) {
		return t == null ? new ErrorManager.Annotation[0] : (ErrorManager.Annotation[])this.annotations_throwable.get(t);
	}

	/* (non-Javadoc)
	 * @see org.openide.ErrorManager#annotate(java.lang.Throwable, int, java.lang.String, java.lang.String, java.lang.Throwable, java.util.Date)
	 */
	public Throwable annotate(
		Throwable t,
		int severity,
		String message,
		String localizedMessage,
		Throwable stackTrace,
		Date date) {
		ErrorManager.Annotation [] currentAnnotations = this.findAnnotations(t);
		ErrorManager.Annotation [] newAnnotations = new ErrorManager.Annotation [currentAnnotations == null ? 1 : (currentAnnotations.length + 1)];
		newAnnotations[0] = new Annotation(t, severity, message, localizedMessage, stackTrace, date);
		if (currentAnnotations != null && currentAnnotations.length > 0)
			System.arraycopy(currentAnnotations, 0, newAnnotations, 1, currentAnnotations.length);
		this.attachAnnotations(t, newAnnotations);
		return t;
	}
	
	private String prepareAnnotations(Throwable t) {
		ErrorManager.Annotation[] anns = this.findAnnotations(t);
		StringBuffer annsS = new StringBuffer();
		if (anns != null && anns.length > 0) {
			for (int i = 0; i < anns.length; ++i) {
				if (i > 0)
					annsS.append(System.getProperty("line.separator", "\n"));
				annsS.append(anns[i].toString());
			}
			return annsS.toString();
		} else
			return "";
	}
	
	protected PrintStream selectStream(int severity) {
		switch (severity) {
			case UNKNOWN:
				return out;
			case INFORMATIONAL:
				return out;
			case WARNING:
				return out;
			case USER:
				return out;
			case EXCEPTION:
				return err;
			case ERROR:
				return err;
			default:
				return out;
		}
	}

	/* (non-Javadoc)
	 * @see org.openide.ErrorManager#notify(int, java.lang.Throwable)
	 */
	public void notify(int severity, Throwable t) {
		this.selectStream(severity).println(t.getMessage());
		String anns = this.prepareAnnotations(t);
		if (anns != null && anns.length() > 0)
			this.selectStream(severity).println(anns);
		if (severity == EXCEPTION) {
			t.printStackTrace(this.err);
		}
	}

	/* (non-Javadoc)
	 * @see org.openide.ErrorManager#log(int, java.lang.String)
	 */
	public void log(int severity, String s) {
		this.selectStream(severity).println(s);
	}

	/* (non-Javadoc)
	 * @see org.openide.ErrorManager#getInstance(java.lang.String)
	 */
	public ErrorManager getInstance(String name) {
		return new MDRLogger(name);
	}

}
