/*
 * Created on 12 juil. 2004
 *
 * TODO To change the template for this generated file go to
 * Window - Preferences - Java - Code Style - Code Templates
 */
package ch.epfl.lgl.mdrdemo;

import java.io.BufferedOutputStream;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.Arrays;
import java.util.Iterator;
import java.util.List;

import javax.jmi.model.ModelPackage;
import javax.jmi.model.MofPackage;
import javax.jmi.reflect.RefPackage;
import javax.jmi.xmi.MalformedXMIException;
import javax.jmi.xmi.XmiReader;
import javax.jmi.xmi.XmiWriter;

import org.netbeans.api.mdr.CreationFailedException;
import org.netbeans.api.mdr.MDRManager;
import org.netbeans.api.mdr.MDRepository;

import ch.epfl.lgl.MDR.RepositoryLookup;

/**
 * A demo class showing how to setup the MDR tool by loading a metamodel.
 */
public class MDRDemo_Initialization {

	public static final XmiReader reader;
	public static final XmiWriter writer;
	public static final MDRManager mdrManager;
	public static final String rootPackage;
	protected static final StringBuffer decompiler;
	static {
		rootPackage = ".";
		
		System.out.println("Setting up MDR.");
		//A simple in-memory standalone repository ; another possibility is to define a serialized one (why not the one used by Poseidon ?)
		System.setProperty("org.openide.util.Lookup", "ch.epfl.lgl.MDR.RepositoryLookup");
		//MDRManager is the entry point for MDR
		mdrManager = org.netbeans.api.mdr.MDRManager.getDefault();
		//Here are the XMI reader and writer
		reader = (XmiReader) RepositoryLookup.getDefault().lookup(XmiReader.class);
		writer = (XmiWriter) RepositoryLookup.getDefault().lookup(XmiWriter.class);
		Runtime.getRuntime().addShutdownHook(new Thread (new Runnable () {
			public void run () {
				System.out.println("Shutting down MDR.");
				//kills the process
				mdrManager.shutdownAll();
			}
		}));
		//Here is just a patch to view the generated classes
		decompiler = new StringBuffer();
		org.netbeans.mdr.handlers.BaseObjectHandler.setClassLoaderProvider(new org.netbeans.mdr.handlers.ClassLoaderProvider () {
			public ClassLoader getClassLoader() {
				return ClassLoader.getSystemClassLoader();
			}
			

			public Class defineClass(String className, byte[] classFile) {
				try {
					final File outFile = new File(rootPackage + "/Generation/MDR/java/bin/" + className.replace('.', '/') + ".class");
					if (! outFile.exists()) {
						System.out.println("Archiving bytecode for class " + className);
						outFile.getParentFile().mkdirs();
						outFile.createNewFile();
						BufferedOutputStream out = new BufferedOutputStream(new FileOutputStream(outFile));
						out.write(classFile);
						out.flush();
						out.close();
					}
					final java.io.File javaOutFile = new java.io.File(rootPackage + "/Generation/MDR/java/src/" + className.replace('.', '/') + ".java");
					if (! javaOutFile.exists()) {
						javaOutFile.getParentFile().mkdirs();
						decompiler.append("jad -o -sjava -d " + javaOutFile.getParent() + ' ' + outFile.getAbsolutePath() + '\n');
					}	
				} catch (Exception x) {
					x.printStackTrace();
				} finally {
				}
				return null;
			}
		});
		
		//Performing decompilation task when exiting the program
		Runtime.getRuntime().addShutdownHook(new Thread(new Runnable() {

			public void run() {
				//Has MDR generated new classes ?
				if (decompiler.length() > 0) {
					try {
						System.out.println("Decompilling files.");
						File decompilerBatch = File.createTempFile("MDRDecompilation", ".bat");
						decompilerBatch.deleteOnExit();
						BufferedOutputStream decompilerStream = new BufferedOutputStream(new FileOutputStream(decompilerBatch));
						//Temporary file comits suicide!
						decompiler.append("del ");
						decompiler.append(decompilerBatch.getAbsolutePath());
						decompilerStream.write(decompiler.toString().getBytes());
		/*/				Runtime.getRuntime().exec(decompilerBatch.getAbsolutePath());//*/
						System.out.println("Please, run " + decompilerBatch.getAbsolutePath() + " manually to decompile files...");//*/
					} catch (Exception x) {}
				}
			}
			
		}));
	}
	
	public static String repositoryName = "MyRepository";
	
	public static MofPackage loadDataBaseMetamodelAndReturnRootDataBasePackage() throws CreationFailedException, IOException, MalformedXMIException {
		//Creates a repository
		MDRepository repository = mdrManager.getRepository(repositoryName);
		System.out.println("Repository " + repositoryName + " retreived.");
		
		//A repository is a set of "extents", each one of them can contain a model 
		//Here is the creation of a MOF-based model (so called a metamodel)...
		//This procedure returns the root package of the extent ; in this case, it is a MOF package
		ModelPackage rootXMLMMPackage = (ModelPackage)repository.createExtent("DataBase_Extent");
		System.out.println("Extent DataBase_Extent created for storing the DataMase metamodel as a MOF model.");
		
		//Then, it is possible to include it an already existing MOF-based (either 1.3 or 1.4) XMI (either 1.1 or 1.2) model
		//Reading returns all "readen" objects
		String uri = new File(rootPackage + "/generation/Objecteering/Models/DataBase_MOF.xml").toURI().toString();
		Iterator it = reader.read(uri, rootXMLMMPackage).iterator();
		System.out.println("XMI for DataBase loaded.");
		
		//Let's "instanciate" this package
		//An instanciation is making from a MOF model (i.e. a metamodel) a new extent able to store a model based on this metamodel
		//Instanciation mechanism is only available on MOF-based models...
		//In this case, we need to find the right package to be instaciated (an extent cannot be instanciated)
		//The package to find is named "DataBase"
		//The package to be retreived should be a MofPackage, that is the JMI specific interface for the Package metametaclass defined in MOF.
		MofPackage metaPackageToInstanciate = null;
		Object o;
		List rp = Arrays.asList(new String [] {"DataBase"});
		while (it.hasNext() && (metaPackageToInstanciate == null)) {
			o = it.next();
			if (	(o instanceof MofPackage)
				&&	((MofPackage)o).getQualifiedName().equals(rp)
				) {
				metaPackageToInstanciate = (MofPackage)o;
			}
		}
		System.out.println("Package DataBase to be instanciated retreived.");
		
		return metaPackageToInstanciate;
	}
	
	/**
	 * Loads the DataBase metamodel and "instanciates" it for creating a model repository, i.e. an extent where model elements conforming to the DataBase metamodel can be stored.
	 * @param name the name of the extent to be created.
	 * @return
	 * @throws CreationFailedException
	 * @throws MalformedXMIException
	 * @throws MalformedXMIException
	 * @throws IOException
	 */
	public static RefPackage getExtent(String name) throws CreationFailedException, IOException, MalformedXMIException {
		MofPackage metaPackageToInstanciate = loadDataBaseMetamodelAndReturnRootDataBasePackage();

		//Retreives the repository created by the loadDataBaseMetamodelAndReturnRootDataBasePackage method
		MDRepository repository = mdrManager.getRepository(repositoryName);
		
		//Now the package is found, let's instanciate it !
		RefPackage extent = repository.createExtent(name, metaPackageToInstanciate);
		System.out.println("Package to instanciate instancied into " + name + " extent.");
		
		//The result should be of type DataBasePackage.
		//However, if the JMI interfaces specific to the DataBase metamodel are not available, we shoud better use the reflective interfaces. 
		return extent;
	}
}
