/*
 * Created on Nov 4, 2003
 *
 * To change the template for this generated file go to
 * Window - Preferences - Java - Code Generation - Code and Comments
 */
package ch.epfl.lgl.mdrdemo;

import java.io.File;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

import javax.jmi.model.ModelElement;
import javax.jmi.model.MofClass;
import javax.jmi.model.Reference;
import javax.jmi.model.StructuralFeature;
import javax.jmi.reflect.RefClass;
import javax.jmi.reflect.RefObject;
import javax.jmi.reflect.RefPackage;

/**
 * A demo class to view how to load an XMI model conforming to a given metamodel.
 */
public class MDRDemo_LoadChampionshipModelAndQueryItByReflectiveJMI extends MDRDemo_Initialization {
	
	public static void main (String [] args) {
		try {
			RefPackage championshipExtent = getExtent("ChampionshipExtent_Extent");
			//Load a DataBase model by means of an XMI file.
			String uri = new File(rootPackage + "/generation/Objecteering/Models/Championship.xml").toURI().toString();
			reader.read(uri, championshipExtent);
			System.out.println("Championship.xml loaded.");
			
			//Get existing meta classes.
			LinkedList classes = new LinkedList();
			gatherAllClasses(championshipExtent, classes);
			Iterator classesI = classes.iterator();
			//For each one of them
			while (classesI.hasNext()) {
				RefClass clazz = (RefClass)classesI.next();
				
				//Get the clazz' "meta-object"
				//The meta-object is the object representing the class at the upper layer.
				//It will an object of kind MofClass, sice we are exploring mow the metamodel, instance of MOF.
				//We are here exploring the DataBase metamodel without any knowledge of it!
				MofClass classMetaObject = (MofClass)clazz.refMetaObject();
				
				//We are only interested in concrete classes
				//isAbstract is a JMI operation from the specific JMI interfaces from MOF (like MofClass).
				//This operation is intended to access the isAbstract propery.
				if (!classMetaObject.isAbstract()) {
				
					//Get its existing attributes features of the class using its meta object.
					//The first step is to get all its supertypes
					//The MOF-defined allSupertypes operation helps here...
					List classSupertypesMetaObjects = classMetaObject.allSupertypes();
					ArrayList classesToExploreMetaObjects = new ArrayList(classSupertypesMetaObjects.size() + 1);
					classesToExploreMetaObjects.add(classMetaObject);
					classesToExploreMetaObjects.addAll(classSupertypesMetaObjects);
					
					//Iteration on the class and all of its supertypes in order to find existing structural features.
					Iterator classesToExploreMetaObjectsI = classesToExploreMetaObjects.iterator();
					LinkedList strucuralFeatureMetaObjects = new LinkedList();
					while (classesToExploreMetaObjectsI.hasNext()) {
						MofClass classToExploreMetaObject = (MofClass) classesToExploreMetaObjectsI.next();
						
						//Iteration on the metaobjects for selecting the ones that are structural features.
						Iterator classContentsMetaObjectsI = classToExploreMetaObject.getContents().iterator();
						while (classContentsMetaObjectsI.hasNext()) {
							ModelElement element = (ModelElement) classContentsMetaObjectsI.next();
							if (element instanceof StructuralFeature) {
								strucuralFeatureMetaObjects.add(element);
							}
						}
					}
					
					//Now, let's iterate on model elements, at the model level, instances of the current class.
					//We use here the allOfClass rather than the allOfType because we are only interested in the instances of clazz, and not in the instances of its subclasses.
					System.out.println("Model elements instances of the metaclass " + classMetaObject.getName());
					Iterator instancesI = clazz.refAllOfClass().iterator();
					while (instancesI.hasNext()) {
						RefObject instance = (RefObject) instancesI.next();
						
						//Let's assume this model element have a name, i.e. the metproperty "name" is defined in its metamodel.
						//This is not sure because we are exploring here the actual metamodel, and no langer the MOF!
						System.out.println("\t\"" + instance.refGetValue("name") + "\" has properties");
						
						Iterator featuresI = strucuralFeatureMetaObjects.iterator();
						while (featuresI.hasNext()) {
							StructuralFeature featureMetaObject = (StructuralFeature) featuresI.next();
							
							//Invoking the feature value
							//We are here using the metaobject to indicate what have to be searched at model level!
							Object value = instance.refGetValue(featureMetaObject);
							
							System.out.print("\t\t" + featureMetaObject.getName() + " of type " + featureMetaObject.getType().getName() + " has value ");
							
							//The obtained value may be multiple; we have to check the metafeature multiplicity.
							Collection values;
							int featureMetaObjectMaxMuiltiplicity = featureMetaObject.getMultiplicity().getUpper();
							//-1 stands for *!
							boolean isCollection = featureMetaObjectMaxMuiltiplicity == -1 || featureMetaObjectMaxMuiltiplicity > 1;
							if (isCollection) {
								values = (Collection)value;
							} else {
								values = Arrays.asList(new Object[] {value});
							}
							
							Iterator valuesI = values.iterator();
							if (isCollection)
								System.out.print('[');
							if (!valuesI.hasNext()) {
								System.out.print("<no value>");
							} else {
								boolean end;
								do {
									value = valuesI.next();
									//The obtained value may be
									//	* nothing, if the lower multiplicity is 0 and upper multiplicity is 1
									//  * an object, if behind a MOF reference,
									//	* or a primitive value like Boolean, Integer, Double or String, if behind a MOF Attribute.
									if (value == null)
										System.out.print("<no value>");
									else if (featureMetaObject instanceof Reference)
										//Let's print its name, then (again assuming there is a metaproperty name in the metamodel)
										//We can also print the name of its class by getting it from the model, asking for its metaobject and requiring the name property.
										//It is sure the name property exists for the class metaobject because it must be an instance of the MOF.
										System.out.print("the model element \"" + ((RefObject)value).refGetValue("name") + "\" of metaclass " + ((RefObject)value).refClass().refMetaObject().refGetValue("name"));
									else
										System.out.print(value);
									end = valuesI.hasNext(); 
									if (end)
										System.out.print("; ");
								} while (end);
							}
							if (isCollection)
								System.out.print(']');
							System.out.println();
						}
					}
				}
			}
		} catch (Exception x) {
			x.printStackTrace();
		}
	}
	
	/**
	 * The purpose is to explore a package and transitively extract all of its classes.
	 * @param packageToExplore the package to be explored
	 * @param classesFound the classes the procedure will find will be here.
	 */
	public static void gatherAllClasses (final RefPackage packageToExplore, List classesFound) {
		//Important note: in MOF 1.4, a class cannot contain another !
		classesFound.addAll(packageToExplore.refAllClasses());
		
		Iterator refPackagesI = packageToExplore.refAllPackages().iterator();
		while (refPackagesI.hasNext()) {
			RefPackage pack = (RefPackage) refPackagesI.next();
			gatherAllClasses(pack, classesFound);
		}
	}
}
